# pylint: disable=too-few-public-methods

"""
    CUSTOM WRITER CLASSES
"""
import os
import json
from datetime import datetime

import boto3


class CustomS3JsonWriter:
    """Class to write files to s3"""

    def __init__(self, bucket, profile_name=None):
        self.bucket = bucket

        self.profile_name = profile_name

        if profile_name is None:
            self.boto3_session = boto3.Session()
        else:
            self.boto3_session = boto3.Session(profile_name=profile_name)

        self.s3_resource = self.boto3_session.resource("s3")

    def write_to_s3(self, json_data, config):
        """
        Construct partitioning and file name conventions in s3
        according to business specifications, and write to S3.
        """
        data_variant = config.get("data_variant").lower()
        collection_name = config.get("collection_name").lower()
        organisation_name = config.get("tenant_name").lower()
        if not (data_variant and collection_name and organisation_name and json_data):
            raise ValueError(
                '"data_variant", "collection_name" or "tenant_name" missing from request'
            )
        date = config.get("date", datetime.now().strftime("%Y_%m_%d"))

        key_path = f"{data_variant}/{organisation_name}/{date}/{collection_name}.json"
        # replace any '-' in the path, just in case; we don't want s3 to panic
        key_path = "_".join(key_path.split("-"))

        print(f"Write path: S3://{self.bucket}/{key_path}")
        self.data = json_data
        self.s3_resource.Object(self.bucket, key_path).put(Body=json.dumps(json_data))



import requests

from xero.auth import OAuth2Credentials

from xero.exceptions import *

from oauth2 import *
import boto3
import ast


def get_auth_token(client_id, token_details, profile_name=None):
    """
    Consumes the client id and the previous auth processes refresh token.
    This returns an authentication token that will last 30 minutes
    to make queries the minute it is used. Or it will expire in 60 days of no use.
    The newly generated last refresh token now needs token stored for
    next use.
    """
    bucket = token_details["bucket_name"]
    token_name = token_details["token_path"]
    local_token_path = token_details["local_token_path"]
    boto3_session = None
    if profile_name is None:
        boto3_session = boto3.Session()
    else:
        boto3_session = boto3.Session(profile_name=profile_name)

    s3_resource = boto3_session.resource("s3")
    s3_resource.Bucket(bucket).download_file(token_name, local_token_path)
    token_file = open(local_token_path, "r")

    auth_token = ast.literal_eval(token_file.read())
    # print(auth_token)
    auth_creds = OAuth2Credentials(client_id, client_secret="", token=auth_token)

    return refresh_auth_token(client_id, auth_creds)


def refresh_auth_token(client_id, auth_creds):
    """
    A simple handle of the token expiration.
    """
    if not auth_creds.expired():
        return auth_creds

    cred = {
        "grant_type": "refresh_token",
        "refresh_token": auth_creds.token["refresh_token"],  #
        "client_id": client_id,
    }
    response = requests.post("https://identity.xero.com/connect/token", cred)
    auth_token = response.json()
    print(auth_token)

    save_auth_token(client_id, auth_token)
    return OAuth2Credentials(client_id, client_secret="", token=auth_token)


def save_auth_token(client_id, auth_token):
    """
    function to persist the latest auth token to s3
    """
    # TODO: pull update_refresh_token from readers into here
    pass
